﻿#include "ChunkSerializer.h"
#include "BinaryReader.h"

namespace edge2 {

// チャンク階層の読み込みを開始します。
void ChunkSerializer::ReadChunks(IChunkSerializable& serializable, BinaryReader& reader, uint32_t length) {
	serializable.WillReadChunks();

	int64_t position = reader.position();
	int64_t end_position = position + length;

	while (position < end_position) {
		uint16_t chunk_id = reader.ReadUInt16();
		uint32_t chunk_length = reader.ReadUInt32();
		uint64_t next_position = position + 2 + 4 + chunk_length;

		// チャンク読み込み
		serializable.ReadChunk(reader, chunk_id, chunk_length);

		int64_t cur_position = reader.position();
		if (next_position != cur_position) {
			// 読み込みがスキップされた可能性があります。
			reader.set_position(next_position);
		}

		position = next_position;
	}

	reader.set_position(end_position);

	serializable.DidReadChunks();
}

}