﻿#pragma once

#include <cstdint>

namespace edge2 {

class BinaryReader;

// チャンク形式を使ってシリアライズを行う場合は、このインターフェースに準拠させます。
class IChunkSerializable {
public:
	IChunkSerializable() {}
	virtual ~IChunkSerializable() {}

// IChunkSerializable
public:
	// チャンクデータの読み込むために呼ばれます。
	virtual void ReadChunk(BinaryReader& reader, uint16_t chunk_id, uint32_t length) = 0;
	// この階層のチャンクデータの読み込むが始まる前に呼ばれます。
	virtual void WillReadChunks() {}
	// この階層のチャンクデータの読み込むが終了した後で呼ばれます。
	virtual void DidReadChunks() {}
};

// Edge2の基本的なデータ構造となるチャンク形式を読み込むための支援クラスです。
class ChunkSerializer {
public:
	// チャンク階層の読み込みを開始します。
	static void ReadChunks(IChunkSerializable& serializable, BinaryReader& reader, uint32_t length);
};

}