﻿#pragma once

#include <cstdint>

namespace edge2 {

// アニメーションの共通パラメーターを扱うクラスです。
class Frame {

public:
	// ディレイの単位
	enum DelayUnits : uint8_t {
		kDelayUnit_60FPS = 0,
		kDelayUnit_100FPS,
		kDelayUnit_1000FPS,
		kDelayUnit_10FPS,
		kDelayUnit_15FPS,
		kDelayUnit_20FPS,
		kDelayUnit_24FPS,
		kDelayUnit_30FPS,
	};

	// フレームの後処理
	enum DisposalMethods : uint8_t {
		// 未指定
		kDisposalMethod_NoSpecified = 0,
		// 状態を残す
		kDisposalMethod_DoNotDispose,
		// 背景色でクリアする
		kDisposalMethod_RestoreToBackgroundColor,
		// 前の状態に戻す
		kDisposalMethod_RestoreToPrevious,
	};

// 構築・消滅
public:
	Frame(void);
	virtual ~Frame(void);

// アクセサ
public:
	// ディレイ値を各単位に変換して取得します。
	int delay(DelayUnits unit);
	// ディレイの素の値（6000FPS単位）を設定します。
	void set_delay(int32_t delay) { delay_ = delay; }
	// 透過フラグを取得します。
	bool transparent() { return transparent_; }
	// 透過フラグを設定します。
	void set_transparent(bool transparent) { transparent_ = transparent; }
	// 後処理を取得します。
	DisposalMethods disposal_method() { return disposal_method_; }
	// 後処理を設定します。
	void set_disposal_method(DisposalMethods disposal_method) { disposal_method_ = disposal_method; }

// 変数
private:
	// ディレイ値(6000FPS, 1秒=6000)
	int32_t delay_;
	// 透過
	bool transparent_;
	// 後処理
	DisposalMethods disposal_method_;
};

}
