﻿#include "Image.h"

namespace edge2 {

Image::Image(void)
	: width_(0)
	, height_(0)
	, byte_per_dot_(0)
{
}


Image::~Image(void) {
}

// メモリを確保します。
bool Image::Create(int width, int height, int bit_number) {
	switch (bit_number) {
	case 1:
	case 4:
	case 8:
		byte_per_dot_ = 1;
		break;
	case 15:
	case 16:
	case 24:
		byte_per_dot_ = 3;
		break;
	default:
		return false;
	}

	int buffer_size = width * height * byte_per_dot_;
	if (buffer_size <= 0) {
		return false;
	}

	width_ = width;
	height_ = height;
	bytes_.resize(buffer_size);

	return true;
}

// 指定した色で塗りつぶします。
void Image::Clear(const DrawBackColor& color) {
	if (byte_per_dot_ == 1) {
		memset(bytes(), color.index, buffer_size());
	} else if (byte_per_dot_ == 3) {
		Rgb* p = reinterpret_cast<Rgb*>(bytes());
		Rgb rgb = color.rgb;
		int size = width_ * height_;
		for (int i = 0; i < size; i++) {
			p[i] = rgb;
		}
	}
}

// 背景色を透過して画像をコピーします。thisとdestImageの画像サイズ、フォーマットが一致している必要があります。
void Image::Draw(Image& destImage, const DrawBackColor& transparentColor) {
	if (width_ != destImage.width_ || height_ != destImage.height_ || byte_per_dot_ != destImage.byte_per_dot_) {
		return;
	}
	if (byte_per_dot_ == 1) {
		uint8_t* dest = destImage.bytes();
		uint8_t* src = bytes();
		uint8_t t = transparentColor.index;
		int size = buffer_size();
		for (int i = 0; i < size; i++) {
			if (src[i] != t) {
				dest[i] = src[i];
			}
		}
	} else if (byte_per_dot_ == 3) {
		Rgb* dest = reinterpret_cast<Rgb*>(destImage.bytes());
		Rgb* src = reinterpret_cast<Rgb*>(bytes());
		Rgb t = transparentColor.rgb;
		int size = width_ * height_;
		for (int i = 0; i < size; i++) {
			if (src[i] != t) {
				dest[i] = src[i];
			}
		}
	}
}

}