﻿#pragma once

#include <vector>
#include <cstdint>
#include "DrawBackColor.h"

namespace edge2 {

// ラスター画像（ドットの集合）1枚を処理するためのクラスです。1ドット1バイトor3バイトの２パターンのみ対応しています。パレットは持ちません。
class Image {

// 構築・消滅
public:
	Image(void);
	~Image(void);

// 操作
public:
	// メモリを確保します。
	bool Create(int width, int height, int bit_number);

// ビットマップ処理
public:
	// 指定した色で塗りつぶします。
	void Clear(const DrawBackColor& color);
	// 背景色を透過して画像をコピーします。thisとdestImageの画像サイズ、フォーマットが一致している必要があります。
	void Draw(Image& destImage, const DrawBackColor& transparentColor);

// アクセサ
public:
	// 横幅を取得します。
	int width() { return width_; }
	// 縦幅を取得します。
	int height() { return height_; }
	// 1ドットに必要なバイト数(1 or 3)を取得します。
	int byte_per_dot() { return byte_per_dot_; }
	// 1行に必要なバイト数(を取得します。
	int byte_per_line() { return width_ * byte_per_dot_; }
	// バッファの先頭を取得します。
	uint8_t* bytes() { return &bytes_[0]; }
	// 指定座標のバッファを取得します。
	uint8_t* bytes(int x, int y) { return bytes() + (y * width_ + x) * byte_per_dot_; }
	// バッファサイズを取得します。
	int buffer_size() { return static_cast<int>(bytes_.size()); }

// 変数
private:
	// 横幅
	int width_;
	// 縦幅
	int height_;
	// バイト配列
	std::vector<uint8_t> bytes_;
	// 1ドットに必要なバイト数(1 or 3)
	int byte_per_dot_;
};

}
