﻿#pragma once

#include <assert.h>
#include <vector>
#include "Item.h"
#include "ItemLink.h"

namespace edge2 {

// Itemのポインタを配列で扱うクラスです。カレントアイテムや検索機能などをまとめて扱います。
template<class T>
class Items {
public:
	Items(void) 
		: current_index_(0) {
	}
	virtual ~Items(void) {
		Destroy();
	}

// アクセサ
public:
	// アイテム数を取得します。
	int size() { return static_cast<int>(items_.size()); }
	// カレントアイテムを取得します。
	T& current() { return *static_cast<T*>(items_[current_index_]); }
	// カレントデータのインデックスを取得します。
	int current_index() { return current_index_; }
	// カレントデータのインデックスを指定します。
	void set_current_index(int index) { current_index_ = index; }
	// 指定されたアイテムが所属するグループの開始・終了インデックスを取得します（end_indexは含まれない）。
	void group_range(int index, int& start_index, int& end_index) {
		start_index = index;
		end_index = index + 1;

		for (int i = index; i >= 1; i--) {
			if (items_[i]->grouped()) {
				start_index = i - 1;
			}
		}

		for (; end_index < static_cast<signed>(items_.size()) && items_[end_index]->grouped(); end_index++) { }
	}
	// リンク情報から検索します。見つからない場合はNULLが戻ります。
	T* Find(ItemLink& link) {

		// インデックスキャッシュがあれば、まずはそれを当たりに行きます。
		uint32_t item_id = link.item_id();
		int i = link.index_cache();
		int size = this->size();
		if (i >= 0 && i < size && items_[i]->item_id() == item_id) {
			return &((*this)[i]);
		}

		// 全検索
		for (int i = 0; i < size; i++) {
			if (items_[i]->item_id() == item_id) {
				link.set_index_cache(i);
				return &((*this)[i]);
			}
		}

		return NULL;
	}
	
// 演算子
public:
	// インデクサ
	T& operator[](int index) { return *static_cast<T*>(items_[index]); }

// 操作
public:
	// アイテムを最後尾へ追加します。
	void PushBack(T* item) { items_.push_back(item); }
	// データを破棄します。
	void Destroy() {
		for (unsigned int i = 0; i < items_.size(); i++) {
			delete items_.at(i);
		}
		items_.clear();
	}

// メンバ変数
private:
	std::vector<Item*> items_;
	int current_index_;
	
};

}