﻿#include "Layer.h"
#include "BinaryReader.h"

namespace edge2 {

Layer::Layer(void)
	: layer_id_(0)
	, visible_(true)
	, locked_(false)
{

}

Layer::~Layer(void) {

}

// チャンクデータの読み込むために呼ばれます。
void Layer::ReadChunk(BinaryReader& reader, uint16_t chunk_id, uint32_t length) {
	switch (chunk_id) {
	case kChunkID_Name:
		// ローダーは未対応
		break;
	case kChunkID_ID:
		layer_id_ = reader.ReadInt32();
		break;
	case kChunkID_Grouped:
		set_grouped(reader.ReadBool());
		break;
	case kChunkID_GroupExpansion:
		// ローダーは未対応
		break;
	case kChunkID_Selected:
		// ローダーは未対応
		break;
	case kChunkID_Visible:
		visible_ = reader.ReadBool();
		break;
	case kChunkID_Image:
		if (image_.buffer_size() == length) {
			reader.Read(image_.bytes(), length);
		}
		break;
	case kChunkID_Lock:
		locked_ = reader.ReadBool();
		break;
	case kChunkID_ImageSelection:
		// ローダーは未対応
		break;
	case kChunkID_ItemBgColor:
		// ローダーは未対応
		break;
	case kChunkID_FrameDelay:
		frame_.set_delay(reader.ReadInt32());
		break;
	case kChunkID_FrameTransparent:
		frame_.set_transparent(reader.ReadBool());
		break;
	case kChunkID_FrameDisposalMethod:
		frame_.set_disposal_method(static_cast<Frame::DisposalMethods>(reader.ReadUInt8()));
		break;
	case kChunkID_FrameUse:
		frame_.set_use(reader.ReadBool());
		break;
	}
}

}