﻿#pragma once

#include "ChunkSerializer.h"
#include "Item.h"
#include "Image.h"
#include "LayerFrame.h"

namespace edge2 {

// 1レイヤーのデータクラスです。
class Layer : public Item, public IChunkSerializable {

public:
	enum ChunkIDs {
		kChunkID_Name = 1000,
		kChunkID_ID,
		kChunkID_Grouped,
		kChunkID_GroupExpansion,
		kChunkID_Selected,

		kChunkID_Visible,
		kChunkID_Image,
		kChunkID_Lock,		
		kChunkID_ImageSelection,
		kChunkID_ItemBgColor,

		kChunkID_FrameDelay = 2000,
		kChunkID_FrameTransparent,
		kChunkID_FrameDisposalMethod,
		kChunkID_FrameUse,
	};

// 構築・消滅
public:
	Layer(void);
	~Layer(void);

// アクセサ
public:
	// 可視フラグを取得します。これがtrueでも必ずしもレイヤーが可視ではないことに注意してください。
	bool visible() { return visible_; }
	// 画像の参照を取得します。
	Image& image() { return image_; }

// Item
public:
	// この要素の識別子を取得します。
	virtual int32_t item_id()  { return layer_id_; }

// IChunkSerializable
public:
	// チャンクデータの読み込むために呼ばれます。
	void ReadChunk(BinaryReader& reader, uint16_t chunk_id, uint32_t length);

// 変数
private:
	// レイヤーID
	uint32_t layer_id_;
	// 可視フラグ
	bool visible_;
	// 画像
	Image image_;
	// ロックフラグ
	bool locked_;
	// アニメーション用のフレーム情報
	LayerFrame frame_;
};

}