﻿#pragma once

#include "ChunkSerializer.h"
#include "Items.h"
#include "ItemLink.h"
#include "Layer.h"
#include "PageFrame.h"

namespace edge2 {

// 1ページのデータクラスです。
class Page : public Item, public IChunkSerializable {
public:

	// レイヤー可視モード
	enum LayerVisibleModes {
		// レイヤー毎に可視を設定する（ただしグループの場合親の影響を受ける）
		kLayerVisibleMode_None = 0,
		// カレントレイヤーのみ
		kLayerVisibleMode_Current,
		// カレントレイヤーと一つ上のレイヤーのみ
		kLayerVisibleMode_CurrentAndUp,
		// カレントレイヤーと一つ下のレイヤーのみ
		kLayerVisibleMode_CurrentAndDown,
		// カレントレイヤー以上のみ
		kLayerVisibleMode_OverCurrent,
		// カレントレイヤー以下のみ
		kLayerVisibleMode_UnderCurrent,
		// カレントレイヤーが所属するグループのみ
		kLayerVisibleMode_CurrentGroup,	// Ver.1.11
	};

	// チャンクID（固定値）
	enum ChunkIDs {
		kChunkID_Name = 1000,
		kChunkID_ID,
		kChunkID_Grouped,
		kChunkID_GroupExpansion,
		kChunkID_Selected,

		kChunkID_Width,
		kChunkID_Height,
		kChunkID_PaletteLinkID,
		kChunkID_PaletteLinkIndex,
		kChunkID_ItemBgColor,

		kChunkID_LayerVisibleMode = 2000,
		kChunkID_LayerNextID,
		kChunkID_LayerCount,
		kChunkID_Layer,
		kChunkID_LayerCurrentIndex,
		
		kChunkID_FrameDelay = 3000,
		kChunkID_FrameTransparent,
		kChunkID_FrameDisposalMethod,
		kChunkID_FrameX,
		kChunkID_FrameY,
		kChunkID_FrameUse,

		kChunkID_TipSize = 4000,
		kChunkID_TipPalette,
	};

// 構築・消滅
public:
	Page(void);
	~Page(void);

// アクセサ
public:
	// 色数が何ビットかどうかを設定します。　※複製です。
	void set_bit_number(uint8_t bit_number) { bit_number_ = bit_number; }
	// 横幅を取得します。
	int width() { return width_; }
	// 縦幅を取得します。
	int height() { return height_; }
	// 固定パレットモード時に使用するパレットリンク情報への参照を取得します。
	ItemLink& palette_link() { return palette_link_; }
	// レイヤー可視モードを取得します。
	LayerVisibleModes layer_visible_mode() { return layer_visible_mode_; }
	// レイヤー可視モード込みで指定したレイヤーが可視かどうかを判定します。
	bool layer_visible(int layer_index);
	// レイヤーコンテナを取得します。（0が最上層）
	Items<Layer>& layers() { return layers_; }
	// アニメーション用のフレーム情報を取得します。
	PageFrame& frame() { return frame_; }

// ツール
public:
	// レイヤーが重なった状態の画像を生成します。
	bool CreateMergedImage(Image& image, const DrawBackColor& backColor);

// Item
public:
	// この要素の識別子を取得します。
	virtual int32_t item_id()  { return page_id_; }

// IChunkSerializable
public:
	// チャンクデータの読み込むために呼ばれます。
	void ReadChunk(BinaryReader& reader, uint16_t chunk_id, uint32_t length);

// 変数
private:
	// 何ビットか？(1,4,8,15,16,24)  ※複製です。
	uint8_t bit_number_;
	// ページID
	int32_t page_id_;
	// 横幅
	int width_;
	// 縦幅
	int height_;
	// 固定パレットモード時に使用するパレットへのリンク
	ItemLink palette_link_;
	// レイヤー可視モード
	LayerVisibleModes layer_visible_mode_;
	// レイヤーコンテナ（0が最上層）
	Items<Layer> layers_;
	// アニメーション用のフレーム情報
	PageFrame frame_;
};

}

