﻿#include "Palette.h"
#include "BinaryReader.h"

namespace edge2 {

Palette::Palette(void)
	: palette_id_(0)
{

}

Palette::~Palette(void) {

};

// 色数を指定してパレットのメモリを確保します。
bool Palette::Create(int color_number) {
	if (color_number < 1 || color_number > 256) {
		return false;
	}

	rgbs_.resize(color_number);

	return true;
}

// チャンクデータの読み込むために呼ばれます。
void Palette::ReadChunk(BinaryReader& reader, uint16_t chunk_id, uint32_t length) {
	switch (chunk_id) {
	case kChunkID_Name:
		// ローダーは未対応
		break;
	case kChunkID_ID:
		palette_id_ = reader.ReadInt32();
		break;
	case kChunkID_Grouped:
		set_grouped(reader.ReadBool());
		break;
	case kChunkID_GroupExpansion:
		// ローダーは未対応
		break;
	case kChunkID_Selected:
		// ローダーは未対応
		break;
	case kChunkID_Rgb:
		if (color_number() * sizeof(Rgb) == length) {
			for (int i = 0; i < color_number(); i++) {
				rgb_palette()[i] = reader.ReadRgb();
			}
		}
		break;
	case kChunkID_Word:
		// 欠番
		break;
	case kChunkID_ColorMask:
		// ローダーは未対応
		break;
	case kChunkID_Alpha:
		// 未対応
		break;
	case kChunkID_ColorValue:
		// ローダーは未対応
		break;
	case kChunkID_ColorSpace:
		// ローダーは未対応
		break;
	case kChunkID_ColorSelection:
		// ローダーは未対応
		break;
	case kChunkID_FrameDelay:
		frame_.set_delay(reader.ReadInt32());
		break;
	case kChunkID_FrameTransparent:
		frame_.set_transparent(reader.ReadBool());
		break;
	case kChunkID_FrameDisposalMethod:
		frame_.set_disposal_method(static_cast<Frame::DisposalMethods>(reader.ReadUInt8()));
		break;
	}
}

}