﻿#pragma once

#include <vector>
#include "ChunkSerializer.h"
#include "Item.h"
#include "Rgb.h"
#include "Frame.h"

namespace edge2 {

// 1パレットのデータクラスです。
class Palette : public Item, public IChunkSerializable {

public:
	// チャンクID（固定値）
	enum ChunkIDs {
		kChunkID_Name = 1000,
		kChunkID_ID,
		kChunkID_Grouped,
		kChunkID_GroupExpansion,
		kChunkID_Selected,

		kChunkID_Rgb,
		kChunkID_Word, // 欠番
		kChunkID_ColorMask,
		kChunkID_Alpha,
		kChunkID_ColorValue,
		kChunkID_ColorSpace,
		kChunkID_ColorSelection,

		kChunkID_FrameDelay = 2000,
		kChunkID_FrameTransparent,
		kChunkID_FrameDisposalMethod,

		kChunkID_CommentFirst = 3000,	// ここから256個
	};

// 構築・消滅
public:
	Palette(void);
	~Palette(void);

// アクセサ
public:
	// パレットの色数を取得します。
	int color_number() { return static_cast<int>(rgbs_.size()); }
	// Rgb値の先頭ポインタを取得します。
	Rgb* rgb_palette() { return &rgbs_[0]; }

// 操作
public:
	// 色数を指定してパレットのメモリを確保します。
	bool Create(int color_number);

// Item
public:
	// この要素の識別子を取得します。
	virtual int32_t item_id()  { return palette_id_; }

// IChunkSerializable
public:
	// チャンクデータの読み込むために呼ばれます。
	void ReadChunk(BinaryReader& reader, uint16_t chunk_id, uint32_t length);

// 変数
public:
	// Rgb値の配列
	std::vector<Rgb> rgbs_;
	// パレットID
	int32_t palette_id_;
	// アニメーション用のフレーム情報
	Frame frame_;
};

}

