﻿#include "BitmapEncoder.h"
#include <stdio.h>

BitmapEncoder::BitmapEncoder(void) {
}

BitmapEncoder::~BitmapEncoder(void) {
}

// edge2::Imageに合わせてWindows Bitmap形式でエンコードします。
bool BitmapEncoder::Write(edge2::Image& image, edge2::Palette& palette, const char* path) {
	FILE* fp = fopen(path, "wb");
	if (!fp) {
		return false;
	}

	int color_number = image.byte_per_dot() == 1 ? 256 : 0; // インデックスカラーなら256色とする
	int byte_per_line = image.byte_per_line();
	while (byte_per_line % 4) {
		byte_per_line++;
	}
	
	// BITMAPFILEHEADERを書き出します。
	BITMAPFILEHEADER bitmap_file_header = {0};
	memcpy(&bitmap_file_header.bfType, "BM", 2);
	bitmap_file_header.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * color_number;
	bitmap_file_header.bfSize = bitmap_file_header.bfOffBits + byte_per_line * image.height();
	fwrite(&bitmap_file_header, sizeof(BITMAPFILEHEADER), 1, fp);

	// BITMAPINFOHEADERを書き出します。
	BITMAPINFOHEADER bitmap_info_header = {0};
	bitmap_info_header.biSize = sizeof(BITMAPINFOHEADER);
	bitmap_info_header.biWidth = image.width();
	bitmap_info_header.biHeight = image.height();
	bitmap_info_header.biPlanes = 1;
	bitmap_info_header.biBitCount = image.byte_per_dot() * 8;
	bitmap_info_header.biCompression = 0; // BI_RGB
	bitmap_info_header.biSizeImage = 0;
	bitmap_info_header.biXPelsPerMeter = 0;
	bitmap_info_header.biYPelsPerMeter = 0;
	bitmap_info_header.biClrUsed = 0;
	bitmap_info_header.biClrImportant = 0;
	fwrite(&bitmap_info_header, sizeof(BITMAPINFOHEADER), 1, fp);

	// 必要に応じてパレットを書き出します。
	if (color_number > 0) {
		for (int i = 0; i < color_number; i++) {
			RGBQUAD rgbquad = {0};
			if (i < palette.color_number()) {
				rgbquad.rgbRed   = palette.rgb_palette()[i].r;
				rgbquad.rgbGreen = palette.rgb_palette()[i].g;
				rgbquad.rgbBlue  = palette.rgb_palette()[i].b;
			}
			fwrite(&rgbquad, sizeof(RGBQUAD), 1, fp);
		}
	}

	// 画像を書き出します（上下反転＆アライメント有り）。
	uint32_t dummy = 0;
	int dummy_size = byte_per_line - image.byte_per_line();
	for (int y = image.height() - 1; y >= 0; y--) {
		fwrite(image.bytes(0, y), 1, image.byte_per_line(), fp);
		fwrite(&dummy, 1, dummy_size, fp);
	}

	fclose(fp);
	return true;
}